@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -33,6 +33,8 @@ public enum FileType
     Bmp,
     Gif,
     Ico,
+    Pcx,
+    Riff,
 
     /** Sony camera raw. */
     Arw,
