@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -22,10 +22,15 @@ package com.drew.imaging;
 
 import com.drew.imaging.bmp.BmpMetadataReader;
 import com.drew.imaging.gif.GifMetadataReader;
+import com.drew.imaging.ico.IcoMetadataReader;
 import com.drew.imaging.jpeg.JpegMetadataReader;
+import com.drew.imaging.pcx.PcxMetadataReader;
 import com.drew.imaging.png.PngMetadataReader;
 import com.drew.imaging.psd.PsdMetadataReader;
+import com.drew.imaging.raf.RafMetadataReader;
 import com.drew.imaging.tiff.TiffMetadataReader;
+import com.drew.imaging.webp.WebpMetadataReader;
+import com.drew.lang.RandomAccessStreamReader;
 import com.drew.lang.StringUtil;
 import com.drew.lang.annotations.NotNull;
 import com.drew.metadata.Directory;
@@ -33,7 +38,7 @@ import com.drew.metadata.Metadata;
 import com.drew.metadata.MetadataException;
 import com.drew.metadata.Tag;
 import com.drew.metadata.exif.ExifIFD0Directory;
-import com.drew.metadata.exif.ExifThumbnailDirectory;
+import com.drew.metadata.file.FileMetadataReader;
 
 import java.io.*;
 import java.util.ArrayList;
@@ -41,17 +46,25 @@ import java.util.Arrays;
 import java.util.Collection;
 
 /**
- * Obtains {@link Metadata} from all supported file formats.
+ * Reads metadata from any supported file format.
  * <p>
- * This class a lightweight wrapper around specific file type processors:
+ * This class a lightweight wrapper around other, specific metadata processors.
+ * During extraction, the file type is determined from the first few bytes of the file.
+ * Parsing is then delegated to one of:
+ *
  * <ul>
  *     <li>{@link JpegMetadataReader} for JPEG files</li>
  *     <li>{@link TiffMetadataReader} for TIFF and (most) RAW files</li>
  *     <li>{@link PsdMetadataReader} for Photoshop files</li>
- *     <li>{@link PngMetadataReader} for BMP files</li>
+ *     <li>{@link PngMetadataReader} for PNG files</li>
  *     <li>{@link BmpMetadataReader} for BMP files</li>
  *     <li>{@link GifMetadataReader} for GIF files</li>
+ *     <li>{@link IcoMetadataReader} for ICO files</li>
+ *     <li>{@link PcxMetadataReader} for PCX files</li>
+ *     <li>{@link WebpMetadataReader} for WebP files</li>
+ *     <li>{@link RafMetadataReader} for RAF files</li>
  * </ul>
+ *
  * If you know the file type you're working with, you may use one of the above processors directly.
  * For most scenarios it is simpler, more convenient and more robust to use this class.
  * <p>
@@ -64,17 +77,6 @@ public class ImageMetadataReader
 {
     /**
      * Reads metadata from an {@link InputStream}.
-     * <p>
-     * The file type is determined by inspecting the leading bytes of the stream, and parsing of the file
-     * is delegated to one of:
-     * <ul>
-     *     <li>{@link JpegMetadataReader} for JPEG files</li>
-     *     <li>{@link TiffMetadataReader} for TIFF and (most) RAW files</li>
-     *     <li>{@link PsdMetadataReader} for Photoshop files</li>
-     *     <li>{@link PngMetadataReader} for PNG files</li>
-     *     <li>{@link BmpMetadataReader} for BMP files</li>
-     *     <li>{@link GifMetadataReader} for GIF files</li>
-     * </ul>
      *
      * @param inputStream a stream from which the file data may be read.  The stream must be positioned at the
      *                    beginning of the file's data.
@@ -83,6 +85,21 @@ public class ImageMetadataReader
      */
     @NotNull
     public static Metadata readMetadata(@NotNull final InputStream inputStream) throws ImageProcessingException, IOException
+    {
+        return readMetadata(inputStream, -1);
+    }
+
+    /**
+     * Reads metadata from an {@link InputStream} of known length.
+     *
+     * @param inputStream a stream from which the file data may be read.  The stream must be positioned at the
+     *                    beginning of the file's data.
+     * @param streamLength the length of the stream, if known, otherwise -1.
+     * @return a populated {@link Metadata} object containing directories of tags with values and any processing errors.
+     * @throws ImageProcessingException if the file type is unknown, or for general processing errors.
+     */
+    @NotNull
+    public static Metadata readMetadata(@NotNull final InputStream inputStream, final long streamLength) throws ImageProcessingException, IOException
     {
         BufferedInputStream bufferedInputStream = inputStream instanceof BufferedInputStream
             ? (BufferedInputStream)inputStream
@@ -99,7 +116,7 @@ public class ImageMetadataReader
             fileType == FileType.Nef ||
             fileType == FileType.Orf ||
             fileType == FileType.Rw2)
-            return TiffMetadataReader.readMetadata(bufferedInputStream);
+            return TiffMetadataReader.readMetadata(new RandomAccessStreamReader(bufferedInputStream, RandomAccessStreamReader.DEFAULT_CHUNK_LENGTH, streamLength));
 
         if (fileType == FileType.Psd)
             return PsdMetadataReader.readMetadata(bufferedInputStream);
@@ -113,22 +130,23 @@ public class ImageMetadataReader
         if (fileType == FileType.Gif)
             return GifMetadataReader.readMetadata(bufferedInputStream);
 
+        if (fileType == FileType.Ico)
+            return IcoMetadataReader.readMetadata(bufferedInputStream);
+
+        if (fileType == FileType.Pcx)
+            return PcxMetadataReader.readMetadata(bufferedInputStream);
+
+        if (fileType == FileType.Riff)
+            return WebpMetadataReader.readMetadata(bufferedInputStream);
+
+        if (fileType == FileType.Raf)
+            return RafMetadataReader.readMetadata(bufferedInputStream);
+
         throw new ImageProcessingException("File format is not supported");
     }
 
     /**
      * Reads {@link Metadata} from a {@link File} object.
-     * <p>
-     * The file type is determined by inspecting the leading bytes of the stream, and parsing of the file
-     * is delegated to one of:
-     * <ul>
-     *     <li>{@link JpegMetadataReader} for JPEG files</li>
-     *     <li>{@link TiffMetadataReader} for TIFF and (most) RAW files</li>
-     *     <li>{@link PsdMetadataReader} for Photoshop files</li>
-     *     <li>{@link PngMetadataReader} for PNG files</li>
-     *     <li>{@link BmpMetadataReader} for BMP files</li>
-     *     <li>{@link GifMetadataReader} for GIF files</li>
-     * </ul>
      *
      * @param file a file from which the image data may be read.
      * @return a populated {@link Metadata} object containing directories of tags with values and any processing errors.
@@ -138,11 +156,14 @@ public class ImageMetadataReader
     public static Metadata readMetadata(@NotNull final File file) throws ImageProcessingException, IOException
     {
         InputStream inputStream = new FileInputStream(file);
+        Metadata metadata;
         try {
-            return readMetadata(inputStream);
+            metadata = readMetadata(inputStream, file.length());
         } finally {
             inputStream.close();
         }
+        new FileMetadataReader().read(file, metadata);
+        return metadata;
     }
 
     private ImageMetadataReader() throws Exception
@@ -166,7 +187,6 @@ public class ImageMetadataReader
     public static void main(@NotNull String[] args) throws MetadataException, IOException
     {
         Collection<String> argList = new ArrayList<String>(Arrays.asList(args));
-        boolean thumbRequested = argList.remove("-thumb");
         boolean markdownFormat = argList.remove("-markdown");
         boolean showHex = argList.remove("-hex");
 
@@ -183,7 +203,7 @@ public class ImageMetadataReader
             File file = new File(filePath);
 
             if (!markdownFormat && argList.size()>1)
-                System.out.printf("\n***** PROCESSING: %s\n%n", filePath);
+                System.out.printf("\n***** PROCESSING: %s%n%n", filePath);
 
             Metadata metadata = null;
             try {
@@ -198,8 +218,8 @@ public class ImageMetadataReader
 
             if (markdownFormat) {
                 String fileName = file.getName();
-                String urlName = StringUtil.urlEncode(fileName);
-                ExifIFD0Directory exifIFD0Directory = metadata.getDirectory(ExifIFD0Directory.class);
+                String urlName = StringUtil.urlEncode(filePath);
+                ExifIFD0Directory exifIFD0Directory = metadata.getFirstDirectoryOfType(ExifIFD0Directory.class);
                 String make = exifIFD0Directory == null ? "" : exifIFD0Directory.getString(ExifIFD0Directory.TAG_MAKE);
                 String model = exifIFD0Directory == null ? "" : exifIFD0Directory.getString(ExifIFD0Directory.TAG_MODEL);
                 System.out.println();
@@ -234,9 +254,8 @@ public class ImageMetadataReader
                                 Integer.toHexString(tag.getTagType()),
                                 tagName,
                                 description);
-                    }
-                    else
-                    {
+                    } else {
+                        // simple formatting
                         if (showHex) {
                             System.out.printf("[%s - %s] %s = %s%n", directoryName, tag.getTagTypeHex(), tagName, description);
                         } else {
@@ -249,16 +268,6 @@ public class ImageMetadataReader
                 for (String error : directory.getErrors())
                     System.err.println("ERROR: " + error);
             }
-
-            if (args.length > 1 && thumbRequested) {
-                ExifThumbnailDirectory directory = metadata.getDirectory(ExifThumbnailDirectory.class);
-                if (directory!=null && directory.hasThumbnailData()) {
-                    System.out.println("Writing thumbnail...");
-                    directory.writeThumbnail(args[0].trim() + ".thumb.jpg");
-                } else {
-                    System.out.println("No thumbnail data exists in this image");
-                }
-            }
         }
     }
 }
