@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -21,10 +21,10 @@
 
 package com.drew.imaging.psd;
 
-import com.drew.lang.RandomAccessFileReader;
-import com.drew.lang.RandomAccessStreamReader;
+import com.drew.lang.StreamReader;
 import com.drew.lang.annotations.NotNull;
 import com.drew.metadata.Metadata;
+import com.drew.metadata.file.FileMetadataReader;
 import com.drew.metadata.photoshop.PsdReader;
 
 import java.io.*;
@@ -40,15 +40,13 @@ public class PsdMetadataReader
     public static Metadata readMetadata(@NotNull File file) throws IOException
     {
         Metadata metadata = new Metadata();
-
-        RandomAccessFile randomAccessFile = new RandomAccessFile(file, "r");
-
+        InputStream stream = new FileInputStream(file);
         try {
-            new PsdReader().extract(new RandomAccessFileReader(randomAccessFile), metadata);
+            new PsdReader().extract(new StreamReader(stream), metadata);
         } finally {
-            randomAccessFile.close();
+            stream.close();
         }
-
+        new FileMetadataReader().read(file, metadata);
         return metadata;
     }
 
@@ -56,7 +54,7 @@ public class PsdMetadataReader
     public static Metadata readMetadata(@NotNull InputStream inputStream)
     {
         Metadata metadata = new Metadata();
-        new PsdReader().extract(new RandomAccessStreamReader(inputStream), metadata);
+        new PsdReader().extract(new StreamReader(inputStream), metadata);
         return metadata;
     }
 }
