@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -24,9 +24,12 @@ package com.drew.metadata.adobe;
 import com.drew.lang.annotations.Nullable;
 import com.drew.metadata.TagDescriptor;
 
+import static com.drew.metadata.adobe.AdobeJpegDirectory.*;
+
 /**
  * Provides human-readable string versions of the tags stored in an AdobeJpegDirectory.
  */
+@SuppressWarnings("WeakerAccess")
 public class AdobeJpegDescriptor extends TagDescriptor<AdobeJpegDirectory>
 {
     public AdobeJpegDescriptor(AdobeJpegDirectory directory)
@@ -38,9 +41,9 @@ public class AdobeJpegDescriptor extends TagDescriptor<AdobeJpegDirectory>
     public String getDescription(int tagType)
     {
         switch (tagType) {
-            case AdobeJpegDirectory.TAG_COLOR_TRANSFORM:
+            case TAG_COLOR_TRANSFORM:
                 return getColorTransformDescription();
-            case AdobeJpegDirectory.TAG_DCT_ENCODE_VERSION:
+            case TAG_DCT_ENCODE_VERSION:
                 return getDctEncodeVersionDescription();
             default:
                 return super.getDescription(tagType);
@@ -50,7 +53,7 @@ public class AdobeJpegDescriptor extends TagDescriptor<AdobeJpegDirectory>
     @Nullable
     private String getDctEncodeVersionDescription()
     {
-        Integer value = _directory.getInteger(AdobeJpegDirectory.TAG_COLOR_TRANSFORM);
+        Integer value = _directory.getInteger(TAG_DCT_ENCODE_VERSION);
         return value == null
                 ? null
                 : value == 0x64
@@ -61,14 +64,9 @@ public class AdobeJpegDescriptor extends TagDescriptor<AdobeJpegDirectory>
     @Nullable
     private String getColorTransformDescription()
     {
-        Integer value = _directory.getInteger(AdobeJpegDirectory.TAG_COLOR_TRANSFORM);
-        if (value==null)
-            return null;
-        switch (value) {
-            case 0: return "Unknown (RGB or CMYK)";
-            case 1: return "YCbCr";
-            case 2: return "YCCK";
-            default: return String.format("Unknown transform (%d)", value);
-        }
+        return getIndexedDescription(TAG_COLOR_TRANSFORM,
+            "Unknown (RGB or CMYK)",
+            "YCbCr",
+            "YCCK");
     }
 }
