@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -27,11 +27,15 @@ import com.drew.lang.annotations.Nullable;
 import com.drew.metadata.TagDescriptor;
 
 import java.io.IOException;
+import java.text.DecimalFormat;
+
+import static com.drew.metadata.photoshop.PhotoshopDirectory.*;
 
 /**
  * @author Yuri Binev
  * @author Drew Noakes https://drewnoakes.com
  */
+@SuppressWarnings("WeakerAccess")
 public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
 {
     public PhotoshopDescriptor(@NotNull PhotoshopDirectory directory)
@@ -43,32 +47,32 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getDescription(int tagType)
     {
         switch (tagType) {
-            case PhotoshopDirectory.TAG_THUMBNAIL:
-            case PhotoshopDirectory.TAG_THUMBNAIL_OLD:
+            case TAG_THUMBNAIL:
+            case TAG_THUMBNAIL_OLD:
                 return getThumbnailDescription(tagType);
-            case PhotoshopDirectory.TAG_URL:
-            case PhotoshopDirectory.TAG_XML:
+            case TAG_URL:
+            case TAG_XML:
                 return getSimpleString(tagType);
-            case PhotoshopDirectory.TAG_IPTC:
+            case TAG_IPTC:
                 return getBinaryDataString(tagType);
-            case PhotoshopDirectory.TAG_SLICES:
+            case TAG_SLICES:
                 return getSlicesDescription();
-            case PhotoshopDirectory.TAG_VERSION:
+            case TAG_VERSION:
                 return getVersionDescription();
-            case PhotoshopDirectory.TAG_COPYRIGHT:
+            case TAG_COPYRIGHT:
                 return getBooleanString(tagType);
-            case PhotoshopDirectory.TAG_RESOLUTION_INFO:
+            case TAG_RESOLUTION_INFO:
                 return getResolutionInfoDescription();
-            case PhotoshopDirectory.TAG_GLOBAL_ANGLE:
-            case PhotoshopDirectory.TAG_GLOBAL_ALTITUDE:
-            case PhotoshopDirectory.TAG_URL_LIST:
-            case PhotoshopDirectory.TAG_SEED_NUMBER:
+            case TAG_GLOBAL_ANGLE:
+            case TAG_GLOBAL_ALTITUDE:
+            case TAG_URL_LIST:
+            case TAG_SEED_NUMBER:
                 return get32BitNumberString(tagType);
-            case PhotoshopDirectory.TAG_JPEG_QUALITY:
+            case TAG_JPEG_QUALITY:
                 return getJpegQualityString();
-            case PhotoshopDirectory.TAG_PRINT_SCALE:
+            case TAG_PRINT_SCALE:
                 return getPrintScaleDescription();
-            case PhotoshopDirectory.TAG_PIXEL_ASPECT_RATIO:
+            case TAG_PIXEL_ASPECT_RATIO:
                 return getPixelAspectRatioString();
             default:
                 return super.getDescription(tagType);
@@ -79,21 +83,21 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getJpegQualityString()
     {
         try {
-            byte[] b = _directory.getByteArray(PhotoshopDirectory.TAG_JPEG_QUALITY);
+            byte[] b = _directory.getByteArray(TAG_JPEG_QUALITY);
+
             if (b == null)
-                return _directory.getString(PhotoshopDirectory.TAG_JPEG_QUALITY);
+                return _directory.getString(TAG_JPEG_QUALITY);
+
             RandomAccessReader reader = new ByteArrayReader(b);
             int q = reader.getUInt16(0); // & 0xFFFF;
             int f = reader.getUInt16(2); // & 0xFFFF;
             int s = reader.getUInt16(4);
 
-            int q1;
-            if (q <= 0xFFFF && q >= 0xFFFD)
-                q1 = q - 0xFFFC;
-            else if (q <= 8)
-                q1 = q + 4;
-            else
-                q1 = q;
+            int q1 = q <= 0xFFFF && q >= 0xFFFD
+                ? q - 0xFFFC
+                : q <= 8
+                    ? q + 4
+                    : q;
 
             String quality;
             switch (q) {
@@ -120,6 +124,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
                 default:
                     quality = "Unknown";
             }
+
             String format;
             switch (f) {
                 case 0x0000:
@@ -129,14 +134,16 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
                     format = "Optimised";
                     break;
                 case 0x0101:
-                    format = "Progressive ";
+                    format = "Progressive";
                     break;
                 default:
                     format = String.format("Unknown 0x%04X", f);
             }
+
             String scans = s >= 1 && s <= 3
                     ? String.format("%d", s + 2)
                     : String.format("Unknown 0x%04X", s);
+
             return String.format("%d (%s), %s format, %s scans", q1, quality, format, scans);
         } catch (IOException e) {
             return null;
@@ -147,7 +154,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getPixelAspectRatioString()
     {
         try {
-            byte[] bytes = _directory.getByteArray(PhotoshopDirectory.TAG_PIXEL_ASPECT_RATIO);
+            byte[] bytes = _directory.getByteArray(TAG_PIXEL_ASPECT_RATIO);
             if (bytes == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(bytes);
@@ -162,7 +169,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getPrintScaleDescription()
     {
         try {
-            byte bytes[] = _directory.getByteArray(PhotoshopDirectory.TAG_PRINT_SCALE);
+            byte bytes[] = _directory.getByteArray(TAG_PRINT_SCALE);
             if (bytes == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(bytes);
@@ -189,13 +196,14 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getResolutionInfoDescription()
     {
         try {
-            byte[] bytes = _directory.getByteArray(PhotoshopDirectory.TAG_RESOLUTION_INFO);
+            byte[] bytes = _directory.getByteArray(TAG_RESOLUTION_INFO);
             if (bytes == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(bytes);
             float resX = reader.getS15Fixed16(0);
             float resY = reader.getS15Fixed16(8); // is this the correct offset? it's only reading 4 bytes each time
-            return resX + "x" + resY + " DPI";
+            DecimalFormat format = new DecimalFormat("0.##");
+            return format.format(resX) + "x" + format.format(resY) + " DPI";
         } catch (Exception e) {
             return null;
         }
@@ -205,7 +213,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getVersionDescription()
     {
         try {
-            final byte[] bytes = _directory.getByteArray(PhotoshopDirectory.TAG_VERSION);
+            final byte[] bytes = _directory.getByteArray(TAG_VERSION);
             if (bytes == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(bytes);
@@ -232,7 +240,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     public String getSlicesDescription()
     {
         try {
-            final byte bytes[] = _directory.getByteArray(PhotoshopDirectory.TAG_SLICES);
+            final byte bytes[] = _directory.getByteArray(TAG_SLICES);
             if (bytes == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(bytes);
@@ -240,16 +248,8 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
             String name = reader.getString(24, nameLength * 2, "UTF-16");
             int pos = 24 + nameLength * 2;
             int sliceCount = reader.getInt32(pos);
-            //pos += 4;
             return String.format("%s (%d,%d,%d,%d) %d Slices",
                     name, reader.getInt32(4), reader.getInt32(8), reader.getInt32(12), reader.getInt32(16), sliceCount);
-            /*for (int i=0;i<sliceCount;i++){
-                pos+=16;
-                int slNameLen=getInt32(b,pos);
-                pos+=4;
-                String slName=new String(b, pos, slNameLen*2,"UTF-16");
-                res+=slName;
-            }*/
         } catch (IOException e) {
             return null;
         }
@@ -263,22 +263,14 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
             if (v == null)
                 return null;
             RandomAccessReader reader = new ByteArrayReader(v);
-            //int pos = 0;
             int format = reader.getInt32(0);
-            //pos += 4;
             int width = reader.getInt32(4);
-            //pos += 4;
             int height = reader.getInt32(8);
-            //pos += 4;
-            //pos += 4; //skip WidthBytes
+            //skip WidthBytes
             int totalSize = reader.getInt32(16);
-            //pos += 4;
             int compSize = reader.getInt32(20);
-            //pos += 4;
             int bpp = reader.getInt32(24);
-            //pos+=2;
-            //pos+=2; //skip Number of planes
-            //int thumbSize=v.length-pos;
+            //skip Number of planes
             return String.format("%s, %dx%d, Decomp %d bytes, %d bpp, %d bytes",
                     format == 1 ? "JpegRGB" : "RawRGB",
                     width, height, totalSize, bpp, compSize);
@@ -291,7 +283,7 @@ public class PhotoshopDescriptor extends TagDescriptor<PhotoshopDirectory>
     private String getBooleanString(int tag)
     {
         final byte[] bytes = _directory.getByteArray(tag);
-        if (bytes == null)
+        if (bytes == null || bytes.length == 0)
             return null;
         return bytes[0] == 0 ? "No" : "Yes";
     }
