@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 Drew Noakes
+ * Copyright 2002-2017 Drew Noakes
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ package com.drew.metadata.exif;
 import com.drew.lang.Rational;
 import org.junit.Test;
 
+import static com.drew.metadata.exif.ExifIFD0Directory.*;
 import static org.junit.Assert.assertEquals;
 
 /**
@@ -37,22 +38,22 @@ public class ExifIFD0DescriptorTest
     public void testXResolutionDescription() throws Exception
     {
         ExifIFD0Directory directory = new ExifIFD0Directory();
-        directory.setRational(ExifIFD0Directory.TAG_X_RESOLUTION, new Rational(72, 1));
+        directory.setRational(TAG_X_RESOLUTION, new Rational(72, 1));
         // 2 is for 'Inch'
-        directory.setInt(ExifIFD0Directory.TAG_RESOLUTION_UNIT, 2);
+        directory.setInt(TAG_RESOLUTION_UNIT, 2);
         ExifIFD0Descriptor descriptor = new ExifIFD0Descriptor(directory);
-        assertEquals("72 dots per inch", descriptor.getDescription(ExifIFD0Directory.TAG_X_RESOLUTION));
+        assertEquals("72 dots per inch", descriptor.getDescription(TAG_X_RESOLUTION));
     }
 
     @Test
     public void testYResolutionDescription() throws Exception
     {
         ExifIFD0Directory directory = new ExifIFD0Directory();
-        directory.setRational(ExifIFD0Directory.TAG_Y_RESOLUTION, new Rational(50, 1));
+        directory.setRational(TAG_Y_RESOLUTION, new Rational(50, 1));
         // 3 is for 'cm'
-        directory.setInt(ExifIFD0Directory.TAG_RESOLUTION_UNIT, 3);
+        directory.setInt(TAG_RESOLUTION_UNIT, 3);
         ExifIFD0Descriptor descriptor = new ExifIFD0Descriptor(directory);
-        assertEquals("50 dots per cm", descriptor.getDescription(ExifIFD0Directory.TAG_Y_RESOLUTION));
+        assertEquals("50 dots per cm", descriptor.getDescription(TAG_Y_RESOLUTION));
     }
 
     @Test
@@ -60,17 +61,17 @@ public class ExifIFD0DescriptorTest
     {
         ExifIFD0Directory directory = ExifReaderTest.processBytes("Tests/Data/windowsXpFields.jpg.app1", ExifIFD0Directory.class);
 
-        assertEquals("Testing artist\0", directory.getString(ExifIFD0Directory.TAG_WIN_AUTHOR, "UTF-16LE"));
-        assertEquals("Testing comments\0", directory.getString(ExifIFD0Directory.TAG_WIN_COMMENT, "UTF-16LE"));
-        assertEquals("Testing keywords\0", directory.getString(ExifIFD0Directory.TAG_WIN_KEYWORDS, "UTF-16LE"));
-        assertEquals("Testing subject\0", directory.getString(ExifIFD0Directory.TAG_WIN_SUBJECT, "UTF-16LE"));
-        assertEquals("Testing title\0", directory.getString(ExifIFD0Directory.TAG_WIN_TITLE, "UTF-16LE"));
+        assertEquals("Testing artist\0", directory.getString(TAG_WIN_AUTHOR, "UTF-16LE"));
+        assertEquals("Testing comments\0", directory.getString(TAG_WIN_COMMENT, "UTF-16LE"));
+        assertEquals("Testing keywords\0", directory.getString(TAG_WIN_KEYWORDS, "UTF-16LE"));
+        assertEquals("Testing subject\0", directory.getString(TAG_WIN_SUBJECT, "UTF-16LE"));
+        assertEquals("Testing title\0", directory.getString(TAG_WIN_TITLE, "UTF-16LE"));
 
         ExifIFD0Descriptor descriptor = new ExifIFD0Descriptor(directory);
-        assertEquals("Testing artist", descriptor.getDescription(ExifIFD0Directory.TAG_WIN_AUTHOR));
-        assertEquals("Testing comments", descriptor.getDescription(ExifIFD0Directory.TAG_WIN_COMMENT));
-        assertEquals("Testing keywords", descriptor.getDescription(ExifIFD0Directory.TAG_WIN_KEYWORDS));
-        assertEquals("Testing subject", descriptor.getDescription(ExifIFD0Directory.TAG_WIN_SUBJECT));
-        assertEquals("Testing title", descriptor.getDescription(ExifIFD0Directory.TAG_WIN_TITLE));
+        assertEquals("Testing artist", descriptor.getDescription(TAG_WIN_AUTHOR));
+        assertEquals("Testing comments", descriptor.getDescription(TAG_WIN_COMMENT));
+        assertEquals("Testing keywords", descriptor.getDescription(TAG_WIN_KEYWORDS));
+        assertEquals("Testing subject", descriptor.getDescription(TAG_WIN_SUBJECT));
+        assertEquals("Testing title", descriptor.getDescription(TAG_WIN_TITLE));
     }
 }
